Description: Add some more Debian architectures to the CPU detection
Author: IOhannes m zmölnig
Origin: Debian
Forwarded: no
Last-Update: 2026-04-23
---
This patch header follows DEP-3: http://dep.debian.net/deps/dep3/
--- juce.orig/extras/Build/CMake/juce_runtime_arch_detection.cpp
+++ juce/extras/Build/CMake/juce_runtime_arch_detection.cpp
@@ -53,7 +53,11 @@
 
 #elif defined(__x86_64) || defined(__x86_64__) || defined(__amd64) || defined(_M_X64)
 
+  #if defined (__ILP32__)
+    #error JUCE_ARCH x32
+  #else
   #error JUCE_ARCH x86_64
+  #endif
 
 #elif defined(__ia64) || defined(__ia64__) || defined(_M_IA64)
 
@@ -70,7 +74,11 @@
 #elif defined(__ppc__) || defined(__ppc) || defined(__powerpc__) || defined(_ARCH_COM) || defined(_ARCH_PWR) || defined(_ARCH_PPC) || defined(_M_MPPC) || defined(_M_PPC)
 
   #if defined(__ppc64__) || defined(__powerpc64__) || defined(__64BIT__)
+   #if __LITTLE_ENDIAN__
+    #error JUCE_ARCH ppc64le
+   #else
     #error JUCE_ARCH ppc64
+   #endif
   #else
     #error JUCE_ARCH ppc
   #endif
@@ -83,6 +91,15 @@
     #error JUCE_ARCH riscv
   #endif
 
+#elif defined(__loongarch64)
+
+  #error JUCE_ARCH loongarch64
+
+#elif defined(__s390x__)
+
+  #error JUCE_ARCH s390x
+
+
 #else
 
   #error JUCE_ARCH unknown
